@@ -32,6 +32,11 @@
 #define __USE_XOPEN_EXTENDED 1
 #include <ftw.h>
 
+#ifndef FTW_CONTINUE
+#define FTW_CONTINUE 0
+#endif
+
+
 static int show_all = 0, header = 0, capabilities = 0, cremove = 0;
 
 static void usage(void)
@@ -49,7 +54,7 @@ static int check_file(const char *fpath,
 	if (S_ISREG(sb->st_mode) == 0)
 		return FTW_CONTINUE;
 
-	int fd = open(fpath, O_RDONLY);
+	int fd = open(fpath, O_RDONLY|O_CLOEXEC);
 	if (fd >= 0) {
 		capng_results_t rc;
 
@@ -84,7 +89,8 @@ static int check_file(const char *fpath,
 //
 int main(int argc, char *argv[])
 {
-#if CAP_LAST_CAP < 31 || !defined (VFS_CAP_U32) || !defined (HAVE_ATTR_XATTR_H)
+#if CAP_LAST_CAP < 31 || !defined (VFS_CAP_U32) || \
+	!defined (HAVE_ATTR_XATTR_H) || !defined(HAVE_SYS_XATTR_H)
 	printf("File based capabilities are not supported\n");
 #else
 	char *path_env, *path = NULL, *dir = NULL;
@@ -100,8 +106,11 @@ int main(int argc, char *argv[])
 					usage();
 			} else if (strcmp(argv[i], "-d") == 0) {
 				for (i=0; i<=CAP_LAST_CAP; i++) {
-					printf("%s\n",
-						capng_capability_to_name(i));
+					const char *n =
+						capng_capability_to_name(i);
+					if (n == NULL)
+						n = "unknown";
+					printf("%s\n", n);
 				}
 				return 0;
 			} else if (argv[i][0] == '/') {
@@ -166,7 +175,7 @@ int main(int argc, char *argv[])
 		check_file(path, &sbuf, 0, NULL);
 	} else if (path && capabilities == 1) {
 		// Write capabilities to file
-		int fd = open(path, O_WRONLY|O_NOFOLLOW);
+		int fd = open(path, O_WRONLY|O_NOFOLLOW|O_CLOEXEC);
 		if (fd < 0) {
 			printf("Could not open %s for writing (%s)\n", path,
 				strerror(errno));
